@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-#
+
 # ======================================================================================================================
-# Copyright (©) 2015-2020 LCS
+# Copyright (©) 2015-2022 LCS
 # Laboratoire Catalyse et Spectrochimie, Caen, France.
 #
 # Authors:
@@ -39,16 +39,29 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-B license and that you accept its terms.
 # ======================================================================================================================
+"""
+SpectroChemPy API.
 
-# We intentionnaly DO NOT SET DOCSTRING, HERE... due to the way doc is
-# builded.
+SpectroChemPy is a framework for processing, analyzing and modeling Spectroscopic data for Chemistry with Python.
+It is a cross platform software, running on Linux, Windows or OS X.
+"""
+
+# check for update
+# import sys
+
+# from time import time
+# from pathlib import Path
+
+__all__ = ["api"]
 
 # import the main api
-from spectrochempy.api import *    # noqa: F403,F401
-from spectrochempy import api
 
-__all__ = api.__all__
+from spectrochempy import api
+from spectrochempy.api import *  # noqa: F401
+
+__all__ += api.__all__
+
 
 # ==============================================================================
-if __name__ == '__main__':
+if __name__ == "__main__":
     pass
