@@ -1,17 +1,36 @@
 import numpy as np
 import re
-from . import NOMASK
-from .misc import TYPE_INTEGER, TYPE_COMPLEX, TYPE_FLOAT
 from colorama import Fore, Style
 
-__all__ = ['numpyprintoptions', 'insert_masked_print',
-           'TBold', 'TRed', 'TGreen', 'TBlue', 'TCyan', 'TMagenta',
-           'TYellow', 'colored', 'colored_output', 'pstr', 'convert_to_html']
+from . import NOMASK
+from .misc import TYPE_INTEGER, TYPE_COMPLEX, TYPE_FLOAT
+
+__all__ = [
+    "numpyprintoptions",
+    "insert_masked_print",
+    "TBold",
+    "TRed",
+    "TGreen",
+    "TBlue",
+    "TCyan",
+    "TMagenta",
+    "TYellow",
+    "colored",
+    "colored_output",
+    "pstr",
+    "convert_to_html",
+]
 
 
 def pstr(object, **kwargs):
-    if hasattr(object, 'implements') \
-            and object.implements() in ['NDArray', 'NDDataset', 'NDPanel', 'Coord', 'CoordSet']:
+    if hasattr(object, "implements") and object.implements() in [
+        "NDArray",
+        "NDComplexArrray",
+        "NDDataset",
+        "LinearCoord",
+        "Coord",
+        "CoordSet",
+    ]:
         return object._cstr(**kwargs).strip()
     else:
         return str(object).strip()
@@ -21,28 +40,37 @@ def pstr(object, **kwargs):
 # Terminal colors and styles
 # ======================================================================================================================
 
-def TBold(text): return Style.BRIGHT + str(text) + Style.RESET_ALL
+
+def TBold(text):
+    return Style.BRIGHT + str(text) + Style.RESET_ALL
 
 
-def TRed(text): return Fore.RED + str(text) + Fore.RESET
+def TRed(text):
+    return Fore.RED + str(text) + Fore.RESET
 
 
-def TGreen(text): return Fore.GREEN + str(text) + Fore.RESET
+def TGreen(text):
+    return Fore.GREEN + str(text) + Fore.RESET
 
 
-def TBlue(text): return Fore.BLUE + str(text) + Fore.RESET
+def TBlue(text):
+    return Fore.BLUE + str(text) + Fore.RESET
 
 
-def TMagenta(text): return Fore.MAGENTA + str(text) + Fore.RESET
+def TMagenta(text):
+    return Fore.MAGENTA + str(text) + Fore.RESET
 
 
-def TYellow(text): return Fore.YELLOW + str(text) + Fore.RESET
+def TYellow(text):
+    return Fore.YELLOW + str(text) + Fore.RESET
 
 
-def TCyan(text): return Fore.CYAN + str(text) + Fore.RESET
+def TCyan(text):
+    return Fore.CYAN + str(text) + Fore.RESET
 
 
-def TBlack(text): return Fore.BLACK + str(text) + Fore.RESET
+def TBlack(text):
+    return Fore.BLACK + str(text) + Fore.RESET
 
 
 def colored(text, color):
@@ -51,7 +79,6 @@ def colored(text, color):
 
 
 def colored_output(out):
-
     regex = r"^(\W*(DIMENSION|DATA).*)$"
     subst = TBold(r"\1")
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
@@ -60,7 +87,7 @@ def colored_output(out):
     subst = TBold(r"\1")
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
-    regex = r'\0{3}([\w\W]*?)\0{3}'
+    regex = r"\0{3}([\w\W]*?)\0{3}"
     subst = TBlack(r"\1")
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
@@ -68,11 +95,11 @@ def colored_output(out):
     subst = TGreen(r"\1") + r"\2"
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
-    regex = r'\0{2}([\w\W]*?)\0{2}'
+    regex = r"\0{2}([\w\W]*?)\0{2}"
     subst = TCyan(r"\1")
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
-    regex = r'\0{1}([\w\W]*?)\0{1}'
+    regex = r"\0{1}([\w\W]*?)\0{1}"
     subst = TBlue(r"\1")
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
@@ -83,24 +110,29 @@ def colored_output(out):
 # html output
 #
 
+
 def html_output(out):
     return out
 
 
 def convert_to_html(obj):
-    tr = "<tr>" \
-         "<td style='padding-right:5px; padding-bottom:0px; padding-top:0px; width:124px'>{0}</td>" \
-         "<td style='text-align:left; padding-bottom:0px; padding-top:0px; {2} '>{1}</td><tr>\n"
+    tr = (
+        "<tr>"
+        "<td style='padding-right:5px; padding-bottom:0px; padding-top:0px; width:124px'>{0}</td>"
+        "<td style='text-align:left; padding-bottom:0px; padding-top:0px; {2} '>{1}</td><tr>\n"
+    )
 
     obj._html_output = True
 
     out = obj._cstr()
 
-    regex = r'\0{3}[\w\W]*?\0{3}'
+    regex = r"\0{3}[\w\W]*?\0{3}"
 
     # noinspection PyPep8
     def subst(match):
-        return "<div>{}</div>".format(match.group(0).replace('\n', '<br/>').replace('\0', ''))
+        return "<div>{}</div>".format(
+            match.group(0).replace("\n", "<br/>").replace("\0", "")
+        )
 
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
@@ -116,35 +148,39 @@ def convert_to_html(obj):
     subst = r"<strong>\1</strong>"
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
-    regex = r'\0{2}[\w\W]*?\0{2}'
+    regex = r"\0{2}[\w\W]*?\0{2}"
 
     # noinspection PyPep8
     def subst(match):
         return "<div><font color='darkcyan'>{}</font></div>".format(
-            match.group(0).replace('\n', '<br/>').replace('\0', ''))
+            match.group(0).replace("\n", "<br/>").replace("\0", "")
+        )
 
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
-    regex = r'\0{1}[\w\W]*?\0{1}'
+    regex = r"\0{1}[\w\W]*?\0{1}"
 
     # noinspection PyPep8
     def subst(match):
         return "<div><font color='blue'>{}</font></div>".format(
-            match.group(0).replace('\n', '<br/>').replace('\0', ''))
+            match.group(0).replace("\n", "<br/>").replace("\0", "")
+        )
 
     out = re.sub(regex, subst, out, 0, re.MULTILINE)
 
-    regex = r'\.{3}\s+\n'
-    out = re.sub(regex, '', out, 0, re.MULTILINE)
+    regex = r"\.{3}\s+\n"
+    out = re.sub(regex, "", out, 0, re.MULTILINE)
 
     html = "<table style='background:transparent'>\n"
     for line in out.splitlines():
-        if '</font> :' in line:
+        if "</font> :" in line:
             # keep only first match
-            parts = line.split(':')
-            html += tr.format(parts[0], ':'.join(parts[1:]), 'border:.5px solid lightgray; ')
-        elif '<strong>' in line:
-            html += tr.format(line, '<hr/>', 'padding-top:10px;')
+            parts = line.split(":")
+            html += tr.format(
+                parts[0], ":".join(parts[1:]), "border:.5px solid lightgray; "
+            )
+        elif "<strong>" in line:
+            html += tr.format(line, "<hr/>", "padding-top:10px;")
     html += "</table>"
 
     obj._html_output = False
@@ -159,11 +195,12 @@ def convert_to_html(obj):
 #  see the header of numpy.ma.core.py for the license
 # ======================================================================================================================
 
+
 class _MaskedPrintOption(object):
+
     # """
     # Handle the string used to represent missing data in a masked array.
     # copied from numpy.ma.core
-    #
     # """
 
     def __init__(self, display):
@@ -277,7 +314,7 @@ def _recursive_printoption(result, mask, printopt):
     return
 
 
-def insert_masked_print(ds, mask_string='--'):
+def insert_masked_print(ds, mask_string="--"):
     """
     Replace masked values with mask_string.
 
@@ -287,7 +324,6 @@ def insert_masked_print(ds, mask_string='--'):
     -----------
     ds : |NDDataset| instance
     mask_string : str
-
     """
 
     mask = ds._mask
@@ -298,8 +334,7 @@ def insert_masked_print(ds, mask_string='--'):
         data = ds._data
         # For big arrays, to avoid a costly conversion to the
         # object dtype, extract the corners before the conversion.
-        print_width = (ds._print_width if ds.ndim > 1
-                       else ds._print_width_1d)
+        print_width = ds._print_width if ds.ndim > 1 else ds._print_width_1d
         for axis in range(ds.ndim):
             if data.shape[axis] > print_width:
                 ind = print_width // 2
@@ -321,10 +356,19 @@ def insert_masked_print(ds, mask_string='--'):
 # numpy printoptions
 # ======================================================================================================================
 
-def numpyprintoptions(precision=4, threshold=6, edgeitems=2, suppress=True,
-                      formatter=None, spc=4, linewidth=150, **kargs):
+
+def numpyprintoptions(
+    precision=4,
+    threshold=6,
+    edgeitems=2,
+    suppress=True,
+    formatter=None,
+    spc=4,
+    linewidth=150,
+    **kargs
+):
     """
-    Method to control array printing
+    Method to control array printing.
 
     Parameters
     ----------
@@ -336,8 +380,6 @@ def numpyprintoptions(precision=4, threshold=6, edgeitems=2, suppress=True,
     spc
     linewidth
     kargs
-
-
     """
 
     def _format_object(x):
@@ -346,45 +388,47 @@ def numpyprintoptions(precision=4, threshold=6, edgeitems=2, suppress=True,
             # a workaround to format masked values
             # the problem is that is depends on the type
             # so we have add a type to the passed object: '--' ->  '--int64'
-            el = str(x)[2:].split('_')
+            el = str(x)[2:].split("_")
             typ = el[0].lower()
-            if 'int' in typ:
-                fmt = "{:>{lspace}s}".format('--', lspace=precision + spc)
-            elif 'float' in typ:
-                fmt = "{:>{lspace}s}".format('--', lspace=2 * precision - 4 + spc)
-            elif 'complex' in typ:
-                fmt = "{:>{lspace}s}".format('--', lspace=4 * precision - 8 + spc)
+            if "int" in typ:
+                fmt = "{:>{lspace}s}".format("--", lspace=precision + spc)
+            elif "float" in typ:
+                fmt = "{:>{lspace}s}".format("--", lspace=2 * precision - 4 + spc)
+            elif "complex" in typ:
+                fmt = "{:>{lspace}s}".format("--", lspace=4 * precision - 8 + spc)
             else:
-                fmt = 'n.d.'
+                fmt = "n.d."
 
         elif isinstance(x, TYPE_FLOAT):
-            fmt = '{:{lspace}.0{prec}g}'.format(
-                x,
-                prec=precision,  # - 1,
-                lspace=precision + spc)
+            fmt = "{:{lspace}.0{prec}g}".format(
+                x, prec=precision, lspace=precision + spc  # - 1,
+            )
 
         elif isinstance(x, TYPE_COMPLEX):
-            fmt = '{:{lspace}.0{prec}g}{:+{lc}.0{prec}g}j'.format(
-                x.real, x.imag, prec=precision - 1,
-                lspace=precision + spc,
-                lc=precision)
+            fmt = "{:{lspace}.0{prec}g}{:+{lc}.0{prec}g}j".format(
+                x.real, x.imag, prec=precision - 1, lspace=precision + spc, lc=precision
+            )
 
         elif isinstance(x, TYPE_INTEGER):
-            fmt = '{:>{lspace}d}'.format(
-                x,
-                lspace=precision + spc)
+            fmt = "{:>{lspace}d}".format(x, lspace=precision + spc)
 
         else:
-            fmt = '  {}'.format(x)
+            fmt = "  {}".format(x)
 
         return fmt
 
     if not formatter:
         spc = 4
         formatter = {
-            'all': _format_object,
+            "all": _format_object,
         }
 
-    np.set_printoptions(precision=precision, threshold=threshold,
-                        edgeitems=edgeitems, suppress=suppress,
-                        formatter=formatter, linewidth=linewidth, **kargs)
+    np.set_printoptions(
+        precision=precision,
+        threshold=threshold,
+        edgeitems=edgeitems,
+        suppress=suppress,
+        formatter=formatter,
+        linewidth=linewidth,
+        **kargs
+    )
